/*****************************************************************************/
/*
   main.cpp
   www.xbdev.net
   Simple Cg demo with DirectX

*/
/*****************************************************************************/

//-----------------------------------------------------------------------------


#pragma comment(lib, "cg.lib")
//#pragma comment(lib, "CgFXParser.lib")
// cgGL.lib, cgD3D8.lib
#pragma comment(lib, "cgD3D9.lib")

#pragma comment(lib, "d3dx9.lib")
#pragma comment(lib, "d3d9.lib")

#define STRICT
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <assert.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <Cg/Cg.h>
#include <Cg/CgD3D9.h>


/*****************************************************************************/
/*
   GLOBALS
   As much as we all hate them, its sometimes a little easier...so we've got
   a few globals for the essential of our code.
/*
/*****************************************************************************/

HWND                    g_hWnd          = NULL;
LPDIRECT3D9             g_pD3D          = NULL;
LPDIRECT3DDEVICE9       g_pd3dDevice    = NULL;
LPDIRECT3DVERTEXBUFFER9 g_pVertexBuffer = NULL;

LPDIRECT3DVERTEXDECLARATION9 g_pVertexDeclaration = NULL;


struct Vertex
{
    float x, y, z;
    DWORD color;

	enum FVF
	{
		FVF_Flags = D3DFVF_XYZ | D3DFVF_DIFFUSE 
	};
};

Vertex g_quadVertices[] =
{
   //  x     y     z       color
   { -1.0f,-1.0f, 0.0f, 0xffffff00, }, // Bottom-Left,  color = yellow
   {  1.0f,-1.0f, 0.0f, 0xff00ff00, }, // Bottom-Right, color = green
   {  1.0f, 1.0f, 0.0f, 0xffff0000, }, // Top-Right,    color = red
   { -1.0f, 1.0f, 0.0f, 0xff0000ff, }  // Top-Left,     color = blue
};

//-----------------------------------------------------------------------------
// PROTOTYPES
//-----------------------------------------------------------------------------
void init(void);         // - called at the start of our program
void render(void);       // - our mainloop..called over and over again
void shutDown(void);     // - last function we call before we end!


/***************************************************************************/
/*                                                                         */
/* Handle all messages for the main window here                            */
/*                                                                         */
/***************************************************************************/
long _stdcall MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if(uMsg == WM_DESTROY)
    {
            // inside init.cpp we destroy any directx memory we allocated etc, 
            // tidy up before leaving.
            // TIDY UP DIRECTX HERE BEFORE EXITING HERE!!!!!!!!!!!!!!!!!!!!!!!!!
            	
		    shutDown();
            //UnregisterClass( "DirectX3D", winClass.hInstance );
            PostQuitMessage(0);
    }
    return (long)DefWindowProc(hWnd, uMsg, wParam, lParam);
}


/***************************************************************************/
/*                                                                         */
/* Program entry point.                                                    */
/*                                                                         */
/***************************************************************************/
int _stdcall WinMain(HINSTANCE i, HINSTANCE, char* k, int) 
{
    MSG msg;
    char szname[] = "DirectX3D";
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      szname, NULL };
    RegisterClassEx( &wc );
    g_hWnd = CreateWindowEx(WS_EX_APPWINDOW,
                              szname, "Basic Bones Sample", 
                              WS_OVERLAPPEDWINDOW,//for fullscreen make into WS_POPUP
                              50, 50, 500,500,    //for full screen GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
                              GetDesktopWindow(), NULL, wc.hInstance, NULL);
    
    // Initilise or directX code here!
    // INIT OUR DIRECTX ONCE HERE AT THE START HERE!!!!!!!!!!!!!!!!!!!!!!!!!
	init();
	//initShader();

    ShowWindow(g_hWnd, SW_SHOW);
    UpdateWindow(g_hWnd);     
 
    // Message loop. Note that this has been modified to allow
    // us to execute if no messages are being processed.
    while(1)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (!GetMessage(&msg, NULL, 0, 0))
                break;
            DispatchMessage(&msg);
        }
        
        // Idle-time processing - call our loop function in game.cpp
        // CALLING OUR DIRECTX CODE IN GAME.CPP FROM HERE!!!!!!!!!!!!!!!!!
        render();
    }
    return 0;
}



/***************************************************************************/
/*                                                                         */
/* init()                                                                  */
/* Called once at the start of our program to setup directx.               */
/*                                                                         */
/***************************************************************************/
void init( void )
{
    g_pD3D = Direct3DCreate9( D3D_SDK_VERSION );

    D3DDISPLAYMODE d3ddm;

    g_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm );

    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof(d3dpp) );

    d3dpp.Windowed               = TRUE;
    d3dpp.SwapEffect             = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat       = d3ddm.Format;
    d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
    d3dpp.PresentationInterval   = D3DPRESENT_INTERVAL_IMMEDIATE;

    g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_hWnd,
                          D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                          &d3dpp, &g_pd3dDevice );
	
	g_pd3dDevice->CreateVertexBuffer( 4*sizeof(Vertex), D3DUSAGE_WRITEONLY, 
		                              Vertex::FVF_Flags, D3DPOOL_DEFAULT, 
                                      &g_pVertexBuffer, NULL );
    void *pVertices = NULL;

    g_pVertexBuffer->Lock( 0, sizeof(g_quadVertices), (void**)&pVertices, 0 );
    memcpy( pVertices, g_quadVertices, sizeof(g_quadVertices) );
    g_pVertexBuffer->Unlock();

	g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
}




//-----------------------------------------------------------------------------
// Name: shutDown()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
void shutDown( void )
{
	g_pVertexBuffer->Release();
    g_pVertexBuffer = NULL;

    g_pd3dDevice->Release();
    g_pd3dDevice = NULL;

    g_pD3D->Release();
    g_pD3D = NULL;
}



void render()
{

	CGcontext myCGcontext = cgCreateContext();

	cgD3D9SetDevice(NULL);
	cgD3D9SetDevice( g_pd3dDevice );

	CGprofile vertexProfile = cgD3D9GetLatestVertexProfile();
	const char **vertexOptions = cgD3D9GetOptimalOptions( vertexProfile );

	CGprogram myCGprogram = cgCreateProgramFromFile( myCGcontext,
											CG_SOURCE,
											"vertex_shader.cg",
											vertexProfile,
											"main",
											vertexOptions );

	cgD3D9LoadProgram( myCGprogram, FALSE, 0 );

	CGparameter myCGparam_worldViewProj = cgGetNamedParameter( myCGprogram, "worldViewProj" );
	CGparameter myCGparam_constColor    = cgGetNamedParameter( myCGprogram, "constColor" );

	assert(cgD3D9TypeToSize(cgGetParameterType(myCGparam_worldViewProj)) == 16 );
	assert(cgD3D9TypeToSize(cgGetParameterType(myCGparam_constColor)) == 4 );

	D3DXVECTOR4 vConstColor( 0.0f, 0.0f, 1.0f, 0.0f );
	cgD3D9SetUniform( myCGparam_constColor, &vConstColor );

	const D3DVERTEXELEMENT9 declaration[] = 
	{
		{ 0, 0 * sizeof(float),D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT,D3DDECLUSAGE_POSITION, 0 },
		{ 0, 3 * sizeof(float),D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR, 0 },
		D3DDECL_END()
	};
	
	assert(cgD3D9ValidateVertexDeclaration( myCGprogram, declaration ));

	g_pd3dDevice->CreateVertexDeclaration( declaration, &g_pVertexDeclaration );
	
	
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
                         D3DCOLOR_COLORVALUE(0.0f,0.0f,0.0f,1.0f), 1.0f, 0 );

    g_pd3dDevice->BeginScene();

	g_pd3dDevice->SetVertexDeclaration( g_pVertexDeclaration );
	cgD3D9BindProgram( myCGprogram );
	
	//Setting Shader Constants
	{
		D3DXMATRIX matTrans;
		D3DXMATRIX matRot;
        D3DXMATRIX matProj,matView,matWorld;

		D3DXMatrixTranslation( &matTrans, 0.0f, 0.0f, 4.0f );
		D3DXMatrixRotationYawPitchRoll( &matRot, 0.0f, 0.0f, 0.0f );
		
		D3DXMatrixPerspectiveFovLH( &matProj, D3DXToRadian( 45.0f ), 
                                640.0f / 480.0f, 0.1f, 100.0f );
	    
		g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );

		matWorld = matRot * matTrans;

		D3DXMatrixIdentity( &matView ); // This sample is not really making use of a view matrix

		D3DXMATRIX worldViewProj = matWorld * matView * matProj;
		D3DXMatrixTranspose( &worldViewProj, &worldViewProj );

		// Load a combined world-view-projection matrix
		cgD3D9SetUniformMatrix( myCGparam_worldViewProj, &worldViewProj );

		//D3DXVECTOR4 vConstColor( 0.0f, 1.0f, 0.0f, 0.0f );
		//cgD3D9SetUniform( g_CGparam_constColor, &vConstColor );
	}

	g_pd3dDevice->SetStreamSource( 0, g_pVertexBuffer, 0, sizeof(Vertex) );
    g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLEFAN, 0, 2 );

    g_pd3dDevice->EndScene();
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );


    //cgD3D9SetDevice(NULL);
	cgDestroyProgram(myCGprogram);
	cgDestroyContext(myCGcontext);

}//End Render()
